-- Create a table to track all players who played in each game
--
-- running time: 1.7 minutes
--
drop table games_lineup cascade constraints;

-- insert the away lineup, including extra record for pitcher (whom we don't know if he's in the batting lineup yet)
create table games_lineup as	(select game_id, away_team_id as team_id, 0 as home_id, 1 as start_fl, 0 as lineup_id,			1 as fld_cd, away_start_pit_id as player_id from games g);
insert into games_lineup	(select game_id, away_team_id as team_id, 0 as home_id, 1 as start_fl, 1 as lineup_id, AWAY_LINEUP1_FLD_CD as fld_cd, AWAY_LINEUP1_BAT_ID as player_id from games g);
insert into games_lineup	(select game_id, away_team_id as team_id, 0 as home_id, 1 as start_fl, 2 as lineup_id, AWAY_LINEUP2_FLD_CD as fld_cd, AWAY_LINEUP2_BAT_ID as player_id from games g);
insert into games_lineup	(select game_id, away_team_id as team_id, 0 as home_id, 1 as start_fl, 3 as lineup_id, AWAY_LINEUP3_FLD_CD as fld_cd, AWAY_LINEUP3_BAT_ID as player_id from games g);
insert into games_lineup	(select game_id, away_team_id as team_id, 0 as home_id, 1 as start_fl, 4 as lineup_id, AWAY_LINEUP4_FLD_CD as fld_cd, AWAY_LINEUP4_BAT_ID as player_id from games g);
insert into games_lineup	(select game_id, away_team_id as team_id, 0 as home_id, 1 as start_fl, 5 as lineup_id, AWAY_LINEUP5_FLD_CD as fld_cd, AWAY_LINEUP5_BAT_ID as player_id from games g);
insert into games_lineup	(select game_id, away_team_id as team_id, 0 as home_id, 1 as start_fl, 6 as lineup_id, AWAY_LINEUP6_FLD_CD as fld_cd, AWAY_LINEUP6_BAT_ID as player_id from games g);
insert into games_lineup	(select game_id, away_team_id as team_id, 0 as home_id, 1 as start_fl, 7 as lineup_id, AWAY_LINEUP7_FLD_CD as fld_cd, AWAY_LINEUP7_BAT_ID as player_id from games g);
insert into games_lineup	(select game_id, away_team_id as team_id, 0 as home_id, 1 as start_fl, 8 as lineup_id, AWAY_LINEUP8_FLD_CD as fld_cd, AWAY_LINEUP8_BAT_ID as player_id from games g);
insert into games_lineup	(select game_id, away_team_id as team_id, 0 as home_id, 1 as start_fl, 9 as lineup_id, AWAY_LINEUP9_FLD_CD as fld_cd, AWAY_LINEUP9_BAT_ID as player_id from games g);
commit;

-- insert the home lineup, including extra record for pitcher (whom we don't know if he's in the batting lineup yet)
insert into games_lineup	(select game_id, home_team_id as team_id, 1 as home_id, 1 as start_fl, 0 as lineup_id,			1 as fld_cd, home_start_pit_id as player_id from games g);
insert into games_lineup	(select game_id, home_team_id as team_id, 1 as home_id, 1 as start_fl, 1 as lineup_id, home_LINEUP1_FLD_CD as fld_cd, home_LINEUP1_BAT_ID as player_id from games g);
insert into games_lineup	(select game_id, home_team_id as team_id, 1 as home_id, 1 as start_fl, 2 as lineup_id, home_LINEUP2_FLD_CD as fld_cd, home_LINEUP2_BAT_ID as player_id from games g);
insert into games_lineup	(select game_id, home_team_id as team_id, 1 as home_id, 1 as start_fl, 3 as lineup_id, home_LINEUP3_FLD_CD as fld_cd, home_LINEUP3_BAT_ID as player_id from games g);
insert into games_lineup	(select game_id, home_team_id as team_id, 1 as home_id, 1 as start_fl, 4 as lineup_id, home_LINEUP4_FLD_CD as fld_cd, home_LINEUP4_BAT_ID as player_id from games g);
insert into games_lineup	(select game_id, home_team_id as team_id, 1 as home_id, 1 as start_fl, 5 as lineup_id, home_LINEUP5_FLD_CD as fld_cd, home_LINEUP5_BAT_ID as player_id from games g);
insert into games_lineup	(select game_id, home_team_id as team_id, 1 as home_id, 1 as start_fl, 6 as lineup_id, home_LINEUP6_FLD_CD as fld_cd, home_LINEUP6_BAT_ID as player_id from games g);
insert into games_lineup	(select game_id, home_team_id as team_id, 1 as home_id, 1 as start_fl, 7 as lineup_id, home_LINEUP7_FLD_CD as fld_cd, home_LINEUP7_BAT_ID as player_id from games g);
insert into games_lineup	(select game_id, home_team_id as team_id, 1 as home_id, 1 as start_fl, 8 as lineup_id, home_LINEUP8_FLD_CD as fld_cd, home_LINEUP8_BAT_ID as player_id from games g);
insert into games_lineup	(select game_id, home_team_id as team_id, 1 as home_id, 1 as start_fl, 9 as lineup_id, home_LINEUP9_FLD_CD as fld_cd, home_LINEUP9_BAT_ID as player_id from games g);
commit;

CREATE INDEX IX_TEMP 
	ON GAMES_LINEUP(GAME_ID, PLAYER_ID)
;
commit;

-- view showing all players listed twice 
create or replace view check_games_lineup_duplicates
as
(
	select game_id, player_id, sum(1) as n 
	from games_lineup 
	group by game_id, player_id
	having sum(1) > 1
);
commit;

-- delete all players that are duplicates
delete from games_lineup g
where exists
	(
		select	1
		from	check_games_lineup_duplicates c
		where	g.game_id = c.game_id
		and	g.player_id = c.player_id
	) 
and g.lineup_id = 0
;
commit;

-- should return 0 records
select count(1) as n
from check_games_lineup_duplicates
;

drop INDEX IX_TEMP
;

-- view showing all sub players once;
-- this is because a player who comes in as a PH 
-- then takes the field appears twice in the SUBS table.
-- I'm selecting min on fld_cd to give him a position other than PH.
-- I'm selecting max on lineup_cd in case the pitcher came into the batting lineup after a DH.
create or replace view subs_unique
as
(
	select s.game_id, s.sub_id as player_id, s.sub_home_id as home_id, max(s.sub_lineup_id) as lineup_id, min(s.sub_fld_cd) as fld_cd, sum(1) as n 
	from subs s left join games_lineup gl
		on s.game_id = gl.game_id
		and s.sub_id = gl.player_id
	where gl.game_id is null
	group by s.game_id, s.sub_id, s.sub_home_id
);
commit;

-- insert the sub players
insert into games_lineup	(select su.game_id, gs.team_id, su.home_id, 0 as start_fl, su.lineup_id, su.fld_cd, su.player_id from subs_unique su left join games_score gs on su.game_id = gs.game_id and su.HOME_ID = gs.home_id);
commit;

ALTER TABLE games_lineup
	ADD CONSTRAINT pk_games_lineup 
	PRIMARY KEY (game_id, player_id)
;

-- should return 0 records
select count(1) as n
from check_games_lineup_duplicates
;

commit;


